# RandomLine.pl
#
# A Movable Type plugin tag for extracting and displaying a random line from
# an MT Module, or from a file.  Note that this randomness operates when 
# you rebuild a template with this tag in it--not when a user refreshes the 
# output page
#
# Author: Martin Sutherland (martin@sunpig.com)
#
# See http://www.sunpig.com/martin/projects/mt/randomline/ for documentation,
# installation instructions, and version history
#
# This is version 1.1 (Feb 2002)
#
# Copyright (c) Sun Pig Enterprises 2002.
#

use strict;
use MT::Template::Context;
MT::Template::Context->add_tag(RandomLine => sub { &randomLine; });


sub randomLine {

	# Get the context object, and the arguments for this tag
	my ($ctx, $args) = @_;

	# Check whether a file or a module has been specified.
	# If neither has been specified, return a blank.  If both have been
	# specified, use the module instead of the file.

	# Get the parameters for the tag
	my $strModule = $args->{'module'} || undef;		# Module name undefined by default
	my $strFile = $args->{'file'} || undef;			# File name undefined by default
	my $allowBlanks = $args->{'allowblanks'} || 0;		# By default, skip blank lines.
	my $num = $args->{'num'} || 1;				# By default, return only one line
	my $allowDuplicates = $args->{'allowduplicates'} || 0;	# By default, don't allow duplicates
	my $glue = $args->{'glue'} || '';			# No default value for glue

	# Check dependencies:

	# 1) There must be either a 'module' or a 'file'
	unless (defined($strFile) || defined($strModule)) { return $ctx->error("Must define either a 'file' or a 'module' parameter for use with <MTRandomLine>"); }
	
	# Load up the lines into an array
	my @lines;
	if (defined($strModule)) {
		# From a module

		# Use the MT::Template class
		require MT::Template;

		# Load up the template module
		my $tmpl = MT::Template->load({ name => $strModule,
                                        blog_id => $ctx->stash('blog_id') })
            		or return $ctx->error("Unable to load the module '$strModule' for <MTRandomLine>");

		# Turn the text in the template into an array of all the lines therein
		my $text = $tmpl->text;

		# Turn the text into an array
		@lines = split /\n/, $text;
		
	} else {
		# From a file

		# Try to open the file
		my $fh;
		open($fh, $strFile) or return $ctx->error("Unable to open the file '$strFile' for <MTRandomLine>");
		
		# Grab each line from the file, and add it to an array.
		my $line;
		while ($line = <$fh>) {
			push @lines, $line;
		}
	}

	
	# By default, the plugin will ignore blank lines in the file from which
	# the random line is to be extracted.  Under some circumstances, though,
	# a blank result may be an acceptable value.  If this isn't the case, 
	# we should remove blank lines from the array.
	
	# While we're at it, we should also strip newline characters from the end of each line.
	my ($line, @stripped);

	while (@lines) {
		$line = shift @lines;

		# strip any newline or carriage return characters
		$line =~ s/(\n|\r)//g;

		unless ( ($line eq "") && !($allowBlanks) ) {
			push @stripped, $line;
		}
	}

	# Count the number of lines in the final array
	my $countLines = @stripped;

	# If duplicates are not allowed, make sure that we're not trying to return more values than there are lines...
	if (($num > $countLines) && (!$allowDuplicates)) { return $ctx->error("There are only $countLines values to select from.  The plugin cannot pick $num distinct values."); }

	# Grab the requisite number of values from the stripped lines array, and 
	# put them into a new array @randomList
	my $i = 0;
	my @randomList = ();
	my $randomLineNumber;
	while ($i < $num) {
		# Get a line, and add it to the randomList array
		$randomLineNumber = int(rand $countLines);
		push(@randomList, $stripped[$randomLineNumber]);
		
		# If duplicates are not allowed, remove the chosen item from the original array, 
		# and re-evaluate the number of elements available to make a selection from
		unless ($allowDuplicates) {
			splice (@stripped, $randomLineNumber, 1);
			$countLines = @stripped;
		}
		
		# Increment the counter
		$i++;
	}

	# "Greater Than" and "Less Than" characters in the glue must come in as HTML entities (&lt; and &gt;),
	# otherwise they mess up Movable Type's parsing.  Here we can decode them back into < and > characters.
	$glue =~ s/\&lt;/</g;
	$glue =~ s/\&gt;/>/g;

	# Return the randomly selected lines, joined by the specified glue.
	return (join($glue, @randomList));
}